package dev.svitan.schemas

import kotlinx.serialization.Serializable
import org.jetbrains.exposed.sql.*
import org.jetbrains.exposed.sql.SqlExpressionBuilder.eq
import org.jetbrains.exposed.sql.transactions.transaction
import java.security.MessageDigest
import java.util.UUID

@Serializable
class AuthDTO(
    val id: String,
    val name: String,
    val createdAt: String,
    val updatedAt: String
)

@Serializable
class NewAuthDTO(
    val name: String,
    val pin: String,
    val key: String
)

class AuthService {
    object Auths : Table("auths") {
        val id = uuid("id").autoGenerate()
        val name = text("name").uniqueIndex()
        val pin = text("pin")
        val key = text("key")
        val createdAt = text("created_at")
        val updatedAt = text("updated_at")

        override val primaryKey = PrimaryKey(id)
    }

    init {
        transaction {
            SchemaUtils.create(Auths)
        }
    }

    fun create(action: NewAuthDTO): UUID = transaction {
        Auths.insert {
            it[name] = action.name
            it[pin] = sha256(action.pin)
            it[key] = sha256(action.key)
            it[createdAt] = ""
            it[updatedAt] = ""
        }[Auths.id]
    }

    fun read(id: UUID): AuthDTO? = transaction {
        Auths.selectAll()
            .where { Auths.id eq id }
            .map {
                AuthDTO(
                    it[Auths.id].toString(),
                    it[Auths.name],
                    it[Auths.createdAt],
                    it[Auths.updatedAt]
                )
            }.singleOrNull()
    }

    fun delete(id: UUID) {
        transaction {
            Auths.deleteWhere { Auths.id eq id }
        }
    }
}

fun sha256(it: String): String {
    val bytes = it.toByteArray()
    val md = MessageDigest.getInstance("SHA-256")
    val digest = md.digest(bytes)
    return digest.joinToString("") { "%02x".format(it) }
}
